//
// parseBots.h, a screensaver by Howard Berkey
//
#ifndef H_PARSEBOT
#define H_PARSEBOT

#include <string>
#include <list>


typedef enum {
	op_nop = 1,		// no operation
	op_add,		// add, arg x + arg y
	op_sub,		// subtract, arg x - arg y
	op_mult,	// mult, argx * argy
	op_div,		// divide, argx / argy
	op_gt,		// test argx > argy
	op_eq,		// test argx == argy
	op_lt,		// test argx < argy
	op_rand,    // arg is mod #
	op_store,	// store arg x in a register y 
	op_clear,	// clear register arg
	op_test,	// if accumulator is set, execute next arg else skip it
	op_branch,	// branch to label
	op_jump,	// jump
	op_return, 	// returns from label
	op_move,	// arg dir (1-360)
	op_scan,	// arg dir (1-360), stores result in accumulator
	op_aim,		// arg dir (1-360)
	op_fire,
	op_mine, 
	op_cyc		// extra cpu cycle
} e_opcode;

class instruction 
{
public:	
	char *m_name;
	e_opcode m_code;
	int m_cycles;
	char m_arg[64];
	char m_arg1[64];
	char m_label[64];
};

typedef list<instruction, allocator<instruction> > program;	


class stackFrame
{
public:
	program::iterator m_pc;	 // pc to return to after branch
};

typedef list<stackFrame, allocator<stackFrame> > pstack;
class RobotThread;

class Robot 
{
public:
	string m_name;
	static float speed;
	static float scanwidth;
	BRect m_bounds;
	void cpu_cycle(RobotThread *); // execute an instruction and draw
	Robot(RobotThread *);
	void interpret(RobotThread *);	// interprets next instruction
	BPoint centroid()
	{
		BPoint p;
		p = m_bounds.RightBottom();
		p.x += m_bounds.LeftTop().x;
		p.y += m_bounds.LeftTop().y;
		p.x /= 2.0;
		p.y /= 2.0;
		return p;
	}	
	void parse(const char *filepath);
protected:

	float p_scan(RobotThread *rt);
	float floatArg(char *arg);
	long longArg(char *arg);	
	void storeArg(char *, float);
	int m_red, m_green, m_blue;
	program m_program;
	program::iterator m_pc; // program counter
	float m_scanning;
	float m_aiming;  // the direction the gun is pointing
	string m_instr;  // current instruction
	pstack m_stack;
	//
	// registers
	//

	float m_accumulator;	// storage for results
	float m_reg1;
	float m_reg2;
	float m_reg3;
	float m_reg4;
};

#endif